/*
 * XYZ Library - cursor.c
 *
 * Copyright (C) 2001 Woodley Packard
 * 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include	<stdio.h>
#include	<stdlib.h>

#include	<X11/Xlib.h>

#include	<yz/window.h>
#include	<yz/draw.h>
#include	<yz/cursor.h>
#include	"window_data.h"

extern window_t	*yz_current_window;
extern window_data	*yz_d;

extern Display		*display;
extern Colormap	cmap;
extern int			screen;

yzShowCursor()
{
	XSetWindowAttributes xa;
	
	if(!yz_current_window)return -1;
	xa.event_mask = XYZ_EVM;
	xa.cursor = yz_d->xcursor;
	XChangeWindowAttributes(display, yz_d->xw, CWEventMask | (yz_d->have_custom_cursor?CWCursor:0), &xa);
}

yzHideCursor()
{
	XSetWindowAttributes xa;
	static Cursor	blankcursor = -1;
	Pixmap	src, mask;
	XColor	fg, bg;
	
	if(!yz_current_window)return -1;
	xa.event_mask = XYZ_EVM;
	if(blankcursor==-1)
	{
		fg.pixel = BlackPixel(display, screen);
		bg.pixel = WhitePixel(display, screen);
		src = XCreatePixmap(display, RootWindow(display, screen), 1, 1, 1);
		mask = XCreatePixmap(display, RootWindow(display, screen), 1, 1, 1);
		blankcursor = XCreatePixmapCursor(display,
    			src, mask, &fg, &bg, 1, 1);
	}
	xa.cursor = blankcursor;
	XChangeWindowAttributes(display, yz_d->xw, CWEventMask | CWCursor, &xa);
}

int	yzSetCursor(cursor_t	*cursor)
{
	XSetWindowAttributes xa;
	Cursor	mycursor = -1;
	Pixmap	src, mask;
	XColor	fg, bg;
	unsigned char	*pixbits, *maskbits;
	unsigned char	bit, mbit;
	int		i, ii, j, W;

	if(!yz_current_window)return -1;

	W = cursor->width;
	W += 7; W = W - (W%8);
	pixbits = malloc(W/8 * cursor->height);
	maskbits = malloc(W/8 * cursor->height);

	for(j=0;j<cursor->height;j++)
		for(i=0;i<W;i+=8)
		{
			bit = mbit = 0;
			for(ii=0;ii<8;ii++)
			{
				bit <<= 1; mbit <<= 1;
				if(i+ii<cursor->width)bit |= cursor->data[(cursor->height-1-j)*cursor->width+i+(7-ii)].r?1:0;
				if(i+ii<cursor->width)mbit |= cursor->data[(cursor->height-1-j)*cursor->width+i+(7-ii)].a?1:0;
			}
			pixbits[(j*W+i)/8] = bit;
			maskbits[(j*W+i)/8] = mbit;
		}

	src = XCreatePixmapFromBitmapData(display, RootWindow(display, screen),
		pixbits, W, cursor->height, 1, 0, 1);
	mask = XCreatePixmapFromBitmapData(display, RootWindow(display, screen),
		maskbits, W, cursor->height, 1, 0, 1);

	xa.event_mask = XYZ_EVM;

	fg.pixel = BlackPixel(display, screen);
	bg.pixel = WhitePixel(display, screen);
	mycursor = XCreatePixmapCursor(display,
    		src, mask, &fg, &bg, cursor->hotx, cursor->hoty);
	xa.cursor = mycursor;
	XChangeWindowAttributes(display, yz_d->xw, CWEventMask | CWCursor, &xa);
	if(yz_d->xcursor>=0)
		XFreeCursor(display, yz_d->xcursor);
	yz_d->xcursor = mycursor;
	
	return 0;
}

int	yzWriteCursor(cursor_t	*crsr, char	*filename)
{
	FILE	*f;
	
	f = fopen(filename, "w");
	if(f <= 0)return -1;
	fwrite(&crsr->width, 4, 4, f);
	fwrite(crsr->data, (crsr->width) * (crsr->height), sizeof(color_t), f);
	fclose(f);
	return 0;
}

cursor_t	*yzLoadCursor(char	*filename)
{
	FILE			*f;
	cursor_t		*crsr;
	unsigned short	x, y;
	
	f = fopen(filename, "r");
	if(f <= 0)return 0;
	crsr = malloc(sizeof(cursor_t));
	if(!crsr){fclose(f);return 0;}
	fread(&crsr->width, 4, 4, f);
	crsr->data = malloc(sizeof(color_t) * crsr->width * crsr->height);
	if(!crsr->data){free(crsr);fclose(f);return 0;}
	fread(crsr->data, (crsr->width) * (crsr->height), sizeof(color_t), f);
	fclose(f);
	return crsr;
}

cursor_t	*yzNewCursor(int	width, int	height)
{
	cursor_t	*crsr;
	
	if(width<0 || height<0)return 0;
	
	crsr = malloc(sizeof(cursor_t));
	if(!crsr)return 0;
	crsr->width = width;
	crsr->height = height;
	crsr->hotx = width/2;
	crsr->hoty = height/2;
	crsr->data = calloc(sizeof(color_t), width * height);
	if(!crsr->data){free(crsr);return 0;}
	return crsr;
}
